;*******************************************************************************
;* DISCLAIMER
;* This software is supplied by Renesas Electronics Corporation and is only
;* intended for use with Renesas products. No other uses are authorized.
;* This software is owned by Renesas Electronics Corporation and is protected
;* under all applicable laws, including copyright laws.
;* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
;* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
;* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
;* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
;* DISCLAIMED.
;* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
;* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
;* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
;* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
;* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
;* Renesas reserves the right, without notice, to make changes to this
;* software and to discontinue the availability of this software.
;* By using this software, you agree to the additional terms and
;* conditions found by accessing the following link:
;* http://www.renesas.com/disclaimer
;*******************************************************************************
;* Copyright (C) 2013 Renesas Electronics Corporation. All rights reserved.
;*******************************************************************************
;* File Name : stl_RL78_sw_crc.asm
;* Version : 1.10
;* Device(s) : RL78/G14
;* Tool-Chain : RL78 CubeSuite+ V2.01.00
;* OS : None
;* H/W Platform : RL78/G14 Target Board
;* Description : RL78 IEC Self Test Library CRC calculation module
;
;      AX     - contains the CRC start value
;      B, DE  - Stores the range from the CRC structure
;      ES HL  - Pointer to the start address & range values in the CRC structure
;
;      AX contains the return value of new calculated crc_value
;
;* Limitations : Any dependancy of the user application in that it might be
;                necessary to dissable interrupts while testing
;                 
;                The module does not perform any result checking. This is left
;                to the application code to confirm that the result is correct
;                and take any necessary corrective action
;*******************************************************************************
;* History :
;* 30.05.2011 V1.00 First Release
;* 15.11.2013 V1.10 Fixed comments
;*******************************************************************************

        NAME    stl_RL78_sw_crc_asm              
        PUBLIC  _stl_RL78_sw_crc_asm

DSEG SADDRP
;******************************************************************************
;* Exported global variables and functions (to be accessed by other files)
;******************************************************************************
crc_input:   DS   2

@@CNST CSEG MIRRORP
               
;******************************************************************************
;* Macro definitions
;******************************************************************************
; CRC16 CCITT Table

crc16_table_asm:
        DW 0000H,01021H,02042H,03063H,04084H,050a5H,060c6H,070e7H,08108H
        DW 09129H,0a14aH,0b16bH,0c18cH,0d1adH,0e1ceH,0f1efH,01231H,00210H
        DW 03273H,02252H,052b5H,04294H,072f7H,062d6H,09339H,08318H,0b37bH
        DW 0a35aH,0d3bdH,0c39cH,0f3ffH,0e3deH,02462H,03443H,00420H,01401H
        DW 064e6H,074c7H,044a4H,05485H,0a56aH,0b54bH,08528H,09509H,0e5eeH
        DW 0f5cfH,0c5acH,0d58dH,03653H,02672H,01611H,00630H,076d7H,066f6H
        DW 05695H,046b4H,0b75bH,0a77aH,09719H,08738H,0f7dfH,0e7feH,0d79dH
        DW 0c7bcH,048c4H,058e5H,06886H,078a7H,00840H,01861H,02802H,03823H
        DW 0c9ccH,0d9edH,0e98eH,0f9afH,08948H,09969H,0a90aH,0b92bH,05af5H
        DW 04ad4H,07ab7H,06a96H,01a71H,00a50H,03a33H,02a12H,0dbfdH,0cbdcH
        DW 0fbbfH,0eb9eH,09b79H,08b58H,0bb3bH,0ab1aH,06ca6H,07c87H,04ce4H
        DW 05cc5H,02c22H,03c03H,00c60H,01c41H,0edaeH,0fd8fH,0cdecH,0ddcdH
        DW 0ad2aH,0bd0bH,08d68H,09d49H,07e97H,06eb6H,05ed5H,04ef4H,03e13H
        DW 02e32H,01e51H,00e70H,0ff9fH,0efbeH,0dfddH,0cffcH,0bf1bH,0af3aH
        DW 09f59H,08f78H,09188H,081a9H,0b1caH,0a1ebH,0d10cH,0c12dH,0f14eH
        DW 0e16fH,01080H,000a1H,030c2H,020e3H,05004H,04025H,07046H,06067H
        DW 083b9H,09398H,0a3fbH,0b3daH,0c33dH,0d31cH,0e37fH,0f35eH,002b1H
        DW 01290H,022f3H,032d2H,04235H,05214H,06277H,07256H,0b5eaH,0a5cbH
        DW 095a8H,08589H,0f56eH,0e54fH,0d52cH,0c50dH,034e2H,024c3H,014a0H
        DW 00481H,07466H,06447H,05424H,04405H,0a7dbH,0b7faH,08799H,097b8H
        DW 0e75fH,0f77eH,0c71dH,0d73cH,026d3H,036f2H,00691H,016b0H,06657H
        DW 07676H,04615H,05634H,0d94cH,0c96dH,0f90eH,0e92fH,099c8H,089e9H
        DW 0b98aH,0a9abH,05844H,04865H,07806H,06827H,018c0H,008e1H,03882H
        DW 028a3H,0cb7dH,0db5cH,0eb3fH,0fb1eH,08bf9H,09bd8H,0abbbH,0bb9aH
        DW 04a75H,05a54H,06a37H,07a16H,00af1H,01ad0H,02ab3H,03a92H,0fd2eH
        DW 0ed0fH,0dd6cH,0cd4dH,0bdaaH,0ad8bH,09de8H,08dc9H,07c26H,06c07H
        DW 05c64H,04c45H,03ca2H,02c83H,01ce0H,00cc1H,0ef1fH,0ff3eH,0cf5dH
        DW 0df7cH,0af9bH,0bfbaH,08fd9H,09ff8H,06e17H,07e36H,04e55H,05e74H
        DW 02e93H,03eb2H,00ed1H,01ef0H


@@CODEL CSEG
;******************************************************************************
;* Outline : stl_RL78_sw_crc_asm - RL78 IEC Software CRC test module
;* Include : none
;* Declaration : stl_RL78_sw_crc_asm:
;* Function Name: stl_RL78_sw_crc_asm
;* Description : IEC Software CRC test code module
;* Argument: AX contains the start_crc_value
;*           B, DE contains the address range contained in the crc structure
;*           ES HL pointer to the start Address contained in the CRC structure
;* Return Value : Register AX - CRC Result 
;* Calling Functions : main.c
;******************************************************************************
; Start of software CRC code
_stl_RL78_sw_crc_asm:

; sort input parameter
        PUSH    HL              ; save base pointer
        PUSH    AX
        MOVW    AX , SP         ; copy stack pointer
        MOVW    HL , AX
        MOV     A , [ HL ]      ; LSB of arg1 
        XCH     A , X
        MOV     A , [ HL + 1 ]  ; MSB of arg1
        MOVW    crc_input , AX         ; move 1st argument ( i )

        
	MOV     A , [ HL + 8 ]  ; LSB of arg2
	XCH     A , X
	MOV     A , [ HL + 9 ]  ; MSB of arg2
	MOVW    HL , AX	
	
	MOV     C, #00H
        MOVW    AX, [HL]              ; lower 16 bit of size parameter
        SUBW    AX, #0001H            ; n-1 effect --- length        
        MOVW    DE, AX                    
        MOVW    AX, [HL+2]            ; upper 16 bit of size parameter 
        XCH     A,X
                               
;check_length not_zero:        
        SUBC    A, #00H              ; n-1 effect --- length
        MOV     B,A                   ; bit 17->20 of size parameter  
        CMP     A, #0FFH             ; check bit 17->20
        BNZ     $length_not_zero
        MOVW    AX,DE
        CMPW    AX,#0FFFFH            ; check bit 00->16
        BZ      $exit                  ; test length = 0

; start crc test
length_not_zero:
        MOVW    AX, [HL+6]            ; bit 17->20 of start address into AX
        XCH     A,  X                 ; bit 17->20 of start address into A
        MOV     ES, A                 ; bit 17->20 of Start address into ES       
        MOVW    AX, [HL+4]            ; start Address into BC (lower 16 bit of address)
        MOVW    HL, AX                ; lower 16 bit of start address into HL  
        CLR1    CY

next_rom:
        MOVW    AX, crc_input       ; after finished parameter sorting load crc input value again into AX
        XCH     A, X                  ; crc>>8 is now in register X
        MOV     A, ES:[HL]            ; load data of ROM/Flash area for new crc calculation into A
        XOR     X, A                  ; result of XOR is address value to catch correct table value
        XCH     A, X                  ; maske in A
        MOV     X, #00H              ; clear register X
        CLR1    CY                    ; Clear Carry Flag
        ROLC    A, 1                  ; multiplication * 2 of address offset value
        XCH     A, X                  ; table address offset value into X of AX register
         ROLC    A, 1                  ; multiplication * 2 of higher part of Address offset value
        ADDW    AX, #crc16_table_asm  ; add onto Table start address calculated offset
        PUSH    HL
        MOVW    HL, AX                ; calculated Table address into HL register
        
; load AX register with calculated table value
        MOV     A, [HL]               ; move first byte ( low byte ) of crc16_table into A
        MOV     X, A                  ; move it into register X ( low byte )
        MOV     A, [HL+1]             ; move second byte ( high byte ) of crc16_table into A
        XCHW    AX, HL                ; move crc16 table value into HL
        MOVW    AX, crc_input       ; reload input crc value for this calculation step into register AX
        MOV     A, #00H              ; clear A, to prepare 8 times left shift of AX which means register X cleared
        XCH     A, X                  ; is equal 8 times left shift of AX registers
        
; XOR calculated crc16_table value with 8 times left shifted input crc value
        XOR     H, A                  ; XOR high bytes
        MOVW    AX, HL                ; new CRC value into AX
        MOVW    crc_input, AX       ; update crc input value to prepare next calculation step if not finished        
        POP     HL
        
; check if all specified Memory is tested
        MOVW    AX, DE                ; move memory_test length indicator into AX
        SUBW    AX, #0001H           ; decrement length indicator
        MOVW    DE, AX                ; restore decremented length indicator in DE
        BNC     $not_finished          ; continue if no underflow of lower 16bit of length indicator 
        MOV     A,  B                 
        SUB     A,  #01H             ; if underflow occured, decrement bit 17->20 of length indicator
        MOV     B,  A
        BC      $exit                  ; if underflow of bit 17->20 of length indicator, calculation finished
        
not_finished:
        MOVW    AX, HL
        ADDW    AX, #0001H           ; next address of checksum calculation ( increment HL )
        MOVW    HL, AX
        BNC     $next_rom
	MOV 	A, ES
        INC     A                    ; if overflow HL, incemnt ES register
	MOV 	ES, A
        BR      $next_rom

; Memory test finished, prepare return value
exit:   MOVW  BC, crc_input                 
	POP AX
	POP HL
        ret
        


        END
